/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2004/08/17 09:46:56 $ $Revision: 1.11 $ */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <gtk/gtk.h>
#include <gtk/gtkgl.h>
#include <sys/timeb.h>
#include "global.h"
#include "io.h"
#include "anim.h"
#include "light.h"

gint AnimframeValue_Changed(GtkWidget *, GtkObject *);
gint PlayPosSlider_Changed(GtkWidget *, GtkWidget *);
int ilog(double);
void perr(char *, ...);
int move_bp(char *,int);
void io_setfiletype(int);
void Menu_Run(GtkWidget *,gpointer);
void Redraw_Light(void);
void Read_Objects(int,int);
void Plot_Objects(void);
void Set_FPos(void);

/* Redraw the GL screen */
gint RedrawGL(GtkWidget *widget)
{
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext)) return(0);
  gdk_gl_drawable_swap_buffers (gldrawable);
  gdk_gl_drawable_gl_end (gldrawable);

  return(1);
}

void Replot_Objects(int mode)
{
  GdkGLContext *glcontext;
  GdkGLDrawable *gldrawable;

  if (mode) anim.objectread=1;
  if ((animation.donotplot) && (!mode)) return;
  if (!anim.running) {
    gtk_widget_show(animation.drawing);
    gtk_main_iteration();
    gtk_main_iteration();
  }
  glcontext = gtk_widget_get_gl_context (gl.glarea);
  gldrawable = gtk_widget_get_gl_drawable (gl.glarea);
  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext)) return;

  glClearColor(gl.bg[0],gl.bg[1],gl.bg[2],gl.bg[3]);
  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
  Redraw_Light();
  Plot_Objects();
  gdk_gl_drawable_gl_end (gldrawable);
  RedrawGL(gl.glarea);
  gtk_widget_hide(animation.drawing);
}

void Change_FramedtMax(double upper)
{
  if (upper<GTK_ADJUSTMENT(anim.animframeslideradj)->value) {
    upper=GTK_ADJUSTMENT(anim.animframeslideradj)->value;
  }
  if (ilog(upper)!=ilog(
		GTK_ADJUSTMENT(anim.animframeslideradj)->upper)) {
    gtk_entry_set_max_length(GTK_ENTRY(anim.animframevalue),
	ilog(upper));
  }
  GTK_ADJUSTMENT(anim.animframeslideradj)->upper=upper;
}

void AdjustAnimFramedt()
{
  int a,dt,dtmax,dtmaxi;
  struct timeb t;
  char s[3];

  /* Too few data, the algorithm would fail */
  if (IMEASUREDTMAX<3) return;

  ftime(&t);
  a=(t.time&8191)*1000+t.millitm;
  dt=a-animation.ltime;
  animation.ltime=a;
  if (dt<0) dt+=8192000;
  /* we assume above 10*slider->upper is not reliable */
  if ((double) dt<10.0*
      (GTK_ADJUSTMENT(anim.animframeslideradj)->upper)) {
    animation.framedtlimitfail=0;
    if (anim.imeasuredt==IMEASUREDTMAX) {
      /* we reached the end of measurement move back data */
      memmove(anim.measuredt,&anim.measuredt[1],
	  (IMEASUREDTMAX-1)*sizeof(int));
      anim.measuredt[IMEASUREDTMAX-1]=dt;
    } else { /* add point */
      anim.measuredt[anim.imeasuredt]=dt;
      anim.imeasuredt++;
    }
    /* measure only if we have enough points */
    if (anim.imeasuredt>IMEASUREDTMAX/2) {
      /* we take out the max so search for it */
      dtmax=0;
      for (a=0;a<anim.imeasuredt;a++) {
	if (anim.measuredt[a]>dtmax) {
	  dtmaxi=a;dtmax=anim.measuredt[a];
	}
      }
      /* Now we average */
      dtmax=0;
      for (a=0;a<anim.imeasuredt;a++) {
	if (a!=dtmaxi) {
	  dtmax+=anim.measuredt[a];
	}
      }
      dtmax/=(anim.imeasuredt-1);
      /* if we are too slow */
      if ((double)dtmax>anim.framedt*1.2) {
	if (anim.adjustframedt) anim.framedt=(int)(dtmax*1.2);
	/* If quad resolution adjustment is permitted try it first */
	if ((anim.adjustquadres) &&
		((anim.framedt>animation.quadadjustdtlim) ||
		 (!anim.adjustframedt))) {
	  /* Check if there is anything to decrease */
	  if (gl.quad.st>animation.quadresmin) {
	    if (gl.quad.st>13) {
	      gl.quad.st=max(gl.quad.st/2,animation.quadresmin);
	    } else {
	      gl.quad.st=max(gl.quad.st*3/4,animation.quadresmin);
	    }
	    gl.quad.sl=gl.quad.st*2;
	    if (anim.adjustframedt) anim.framedt=animation.quadadjustdtlim;
	    sprintf(s,"%d",min(99,gl.quad.st));
	    gtk_entry_set_text(GTK_ENTRY(gl.quadstw), s);
	  }
	}
	if ((double)anim.framedt>
	    GTK_ADJUSTMENT(anim.animframeslideradj)->upper*0.8) {
	  Change_FramedtMax(anim.framedt*2.0);
	}
	GTK_ADJUSTMENT(anim.animframeslideradj)->value= anim.framedt;
	gtk_signal_emit_by_name (GTK_OBJECT (anim.animframeslideradj),
    							"value_changed");
      }
    }
  } else {
    animation.framedtlimitfail++;
    if (animation.framedtlimitfail>2) {
      Change_FramedtMax(
	  GTK_ADJUSTMENT(anim.animframeslideradj)->upper*2.0);
      animation.framedtlimitfail=0;
    }
  }
}

gint AnimateBody(gpointer p)
{
  int rev;

  if ((!anim.running) && (anim.timeouttag)) {
    if ((anim.adjustframedt) || (anim.adjustquadres)) AdjustAnimFramedt();
    if (ll.redraw) {
      ll.redraw=0;
    } else {
      Replot_Objects(0);
    }
    return(1);
  }
  rev=0;
  if (io.in==NULL) return(0);
  animation.framerepeatcounter++;
  Set_FPos();
  if (animation.framerepeatcounter<animation.framerepeat) {
    Replot_Objects(0);
    return(1);
  }
  animation.framerepeatcounter=0;
  animation.framepc+=anim.frameskip*anim.ff*(1-(animation.playdir/2)*2);
  if (animation.framepc<animation.marker1) {
    animation.framepc=(animation.marker2/anim.frameskip)*anim.frameskip;
    rev=1;
  }
  if (animation.framepc>animation.marker2) {
    animation.framepc=animation.marker1; rev=1;
  }
  if ((rev) && (animation.playdir&1)) {
    Menu_Run(NULL, (gpointer) 1);
  }
  GTK_ADJUSTMENT(animation.playposslideradj)->value=(gfloat)animation.framepc;
  gtk_signal_emit_by_name(GTK_OBJECT (animation.playposslideradj),
      							"value_changed");
  return(1);
}

gint Animation(gpointer p)
{
  GtkWidget *widget;

  widget=p;
  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
  if (((anim.adjustframedt) || (anim.adjustquadres)) && (anim.running))
    							AdjustAnimFramedt();
  animation.framerepeatcounter=0;
  Read_Objects(animation.framepc,0);
  Replot_Objects(0);

  return(1);
}

/* EOF */
