/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2004/08/17 09:46:56 $ $Revision: 1.9 $ */
/* Gl Event handler */
#include <ctype.h>
#include <math.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtk/gtkgl.h>
#include "global.h"
#include "light.h"

/* prototypes */
gint Quit(GtkWidget *);
void Menu_Run(GtkWidget *,gpointer);
void Pause_Signal(GtkWidget *,gpointer);
gint RedrawGL(GtkWidget *);
gint AnimateBody(gpointer);
void Replot_Objects(int);
int Move_Cut(double);
void SetNumChange(int);
void SetOriginalNum(int);
void ShowValueChange(int);

void Gl_Key_Event(GtkWidget *widget,GdkEvent *event)
{
  int ch,rp;
  char s[12];
  double egy;

  if (anim.depth) {
    ch=(int)event->key.keyval;
    switch (ch) {
      case 'q' :
	/* quit only for ctrl-q */
	if ((event->key.length==1) && (event->key.string[0]==17)) {
	  Quit(widget);
	}
	break;
      case ' ':
      case '=':
	SetOriginalNum(2);
	break;
      case 'd':
	anim.depth=0;
	ShowValueChange(0);
	break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
	SetNumChange(ch-'0');
	break;
    }
  } else {
    egy=1.0;
    ch=(int)event->key.keyval;
    rp=0;
    switch (ch) {	/* check just normal keys */
      case 'q' :
	/* quit only for ctrl-q */
	if ((event->key.length==1) && (event->key.string[0]==17)) {
	  Quit(widget);
	}
	break;
      case 'r':
	Menu_Run(widget,(gpointer) 0);
	break;
      case 's':
	Menu_Run(widget,(gpointer) 1);
	break;
      case 'p':
	Pause_Signal(widget,(gpointer) 0);
	break;
      case 'd':
	anim.depth=1;
	SetOriginalNum(anim.depth);
	ShowValueChange(1);
	break;
      case 'k':
	egy=-1.0;
      case 'm':
	gl.phix=fmod(gl.phix-egy*anim.keyf*sgn(fabs(gl.phiz)-90.0),360.0);
	sprintf(s,"%.2lf",gl.phix);
	gtk_entry_set_text(GTK_ENTRY(gl.wphix),s);
	rp=1;
	break;
      case 'j':
	egy=-1.0;
      case 'l':
	gl.phiz=fmod(gl.phiz-egy*anim.keyf,360.0);
	sprintf(s,"%.2lf",gl.phiz);
	gtk_entry_set_text(GTK_ENTRY(gl.wphiz),s);
	rp=1;
	break;
      case 'J':
	egy=-1.0;
      case 'L':
	gl.x+=egy*anim.keyf/100.0;
	sprintf(s,"%.2lf",gl.x);
	gtk_entry_set_text(GTK_ENTRY(gl.wx),s);
	rp=1;
	break;
      case 'M':
	egy=-1.0;
      case 'K':
	gl.y+=egy*anim.keyf/100.0;
	sprintf(s,"%.2lf",gl.y);
	gtk_entry_set_text(GTK_ENTRY(gl.wy),s);
	rp=1;
	break;
      case 'i':
	egy=-1.0;
      case 'n':
	gl.phiy=fmod(gl.phiy-egy*anim.keyf,360.0);
	sprintf(s,"%.2lf",gl.phiy);
	gtk_entry_set_text(GTK_ENTRY(gl.wphiy),s);
	rp=1;
	break;
      case 'I':
	egy=-1.0;
      case 'N':
	gl.z+=egy*anim.keyf/100.0;
	sprintf(s,"%.2lf",gl.z);
	gtk_entry_set_text(GTK_ENTRY(gl.wz),s);
	rp=1;
	break;
    }
  }
  if ((rp) && (!anim.running)) {
    Replot_Objects(0);
  }
}

void Gl_Button_Event(GtkWidget *widget,GdkEvent *event, gpointer data)
{
  if (event->type==GDK_BUTTON_PRESS) {
    gl.cx=event->button.x;
    gl.cy=event->button.y;
    gl.button=event->button.button;
  }
  if (event->type==GDK_BUTTON_RELEASE) {
    /* remove timeouttag if not running */
    if ((!anim.running) && (anim.timeouttag)) {
      gtk_timeout_remove(anim.timeouttag);
      anim.timeouttag=0;
    }
  }
}


void Gl_Motion_Event(GtkWidget *widget,GdkEvent *event, gpointer data)
{
  double dx,dy;
  double x,y,z,a;
  char s[9];
  int ok;

  dx=event->motion.x-gl.cx;
  dy=event->motion.y-gl.cy;
  gl.cx=event->motion.x;
  gl.cy=event->motion.y;
  if (ll.moveit) {	/* move light instead of objects */
    ll.redraw=1;
    switch (gl.button) {
      case 1:
	x=gl.light[ll.light].param[0][0];
	y=gl.light[ll.light].param[0][1];
	y-=dy/100.0;
	x+=dx/100.0;
	if (fabs(x)>9999.99) x=9999.99*(x/fabs(x));
	if (fabs(y)>9999.99) y=9999.99*(y/fabs(y));
	if (ll.param==0) {
	  sprintf(s,"%.2lf",x);
	  gtk_entry_set_text(GTK_ENTRY(ll.r),s);
	  sprintf(s,"%.2lf",y);
	  gtk_entry_set_text(GTK_ENTRY(ll.g),s);
	}
	gl.light[ll.light].param[0][0]=x;
	gl.light[ll.light].param[0][1]=y;
	break;
      case 2:
	z=gl.light[ll.light].param[0][2];
	z+=dx/100.0;
	z+=dy/100.0;
	if (fabs(z)>9999.99) z=9999.99*(z/fabs(z));
	sprintf(s,"%.2lf",z);
	if (ll.param==0) gtk_entry_set_text(GTK_ENTRY(ll.b),s);
	gl.light[ll.light].param[0][2]=z;
	break;
      case 3:
	a=gl.light[ll.light].param[0][3];
	a+=dy/100.0;
	a+=dx/100.0;
	if (a<0.0) a=0.0;
	if (a>1.0) a=1.0;
	sprintf(s,"%.2lf",a);
	if (ll.param==0) gtk_entry_set_text(GTK_ENTRY(ll.a),s);
	gl.light[ll.light].param[0][3]=a;
	break;
    }
  } else {
    ok=0;
    if (anim.depth) {
      ok=Move_Cut(dy);
    }
    if (!ok) {
      anim.depth=0;
      switch (gl.button) {
	case 1:
	  /* this is to invert up/down tilt if we look at from the other side */
	  gl.phix=fmod(gl.phix-dy*sgn(fabs(gl.phiz)-90.0),360.0);
/*	gl.phix=fmod(gl.phix+dy,360.0);*/
	  gl.phiz=fmod(gl.phiz-dx,360.0);
	  sprintf(s,"%.2lf",gl.phix);
	  gtk_entry_set_text(GTK_ENTRY(gl.wphix),s);
	  sprintf(s,"%.2lf",gl.phiz);
	  gtk_entry_set_text(GTK_ENTRY(gl.wphiz),s);
	  break;
	case 2:
	  gl.y-=dy/100.0;
	  gl.x+=dx/100.0;
	  sprintf(s,"%.2lf",gl.x);
	  gtk_entry_set_text(GTK_ENTRY(gl.wx),s);
	  sprintf(s,"%.2lf",gl.y);
	  gtk_entry_set_text(GTK_ENTRY(gl.wy),s);
	  break;
	case 3:
	  gl.phiy+=dx/3;
	  gl.phiy=fmod(gl.phiy+dx,360.0);
	  gl.z-=dy/100.0;
	  sprintf(s,"%.2lf",gl.phiy);
	  gtk_entry_set_text(GTK_ENTRY(gl.wphiy),s);
	  sprintf(s,"%.2lf",gl.z);
	  gtk_entry_set_text(GTK_ENTRY(gl.wz),s);
      }
    }
  }
  if (anim.detach) {
    Replot_Objects(0);
  } else if (!anim.running) {
    /* If animation is not running we have to create a timeouttag */
    if (!anim.timeouttag) {
      anim.timeouttag=gtk_timeout_add(anim.framedt,AnimateBody,
		      GTK_OBJECT(gl.glarea));
    }
  }
}

void ResizeGL(GtkWidget *widget,GdkEvent *event)
{
  int height,width;
  double mult;
  char s[9];
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext)) return;
  height=max(event->configure.height,1);
  width=max(event->configure.width,1);
  glViewport(0,0,width,height);
  gl.persp.aspect=(double)width/height;
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(gl.persp.angle,gl.persp.aspect,
      gl.persp.zNear,gl.persp.zFar);
  glMatrixMode(GL_MODELVIEW);
  if (gl.height>0) {
    mult=(double)height/gl.height;
    gl.x*=mult;
    gl.y*=mult;
    gl.z*=mult;
    sprintf(s,"%.2lf",gl.x);
    gtk_entry_set_text(GTK_ENTRY(gl.wx),s);
    sprintf(s,"%.2lf",gl.y);
    gtk_entry_set_text(GTK_ENTRY(gl.wy),s);
    sprintf(s,"%.2lf",gl.z);
    gtk_entry_set_text(GTK_ENTRY(gl.wz),s);
  }
  gl.width=width;
  gl.height=height;
  gdk_gl_drawable_gl_end (gldrawable);
  Replot_Objects(0);
}

/* Initializes event callbacks */
void Gl_Window_Events(GtkWidget *glwin)
{
  gtk_widget_add_events(glwin, 	GDK_BUTTON_PRESS_MASK |
      				GDK_BUTTON_RELEASE_MASK |
				GDK_BUTTON1_MOTION_MASK |
				GDK_BUTTON2_MOTION_MASK |
				GDK_BUTTON3_MOTION_MASK);
  gtk_signal_connect (GTK_OBJECT (glwin), "key_press_event",
		      GTK_SIGNAL_FUNC (Gl_Key_Event), NULL);
  gtk_signal_connect (GTK_OBJECT (glwin), "button_press_event",
		      GTK_SIGNAL_FUNC (Gl_Button_Event), NULL);
  gtk_signal_connect (GTK_OBJECT (glwin), "button_release_event",
		      GTK_SIGNAL_FUNC (Gl_Button_Event), NULL);
  gtk_signal_connect (GTK_OBJECT (glwin), "motion_notify_event",
		      GTK_SIGNAL_FUNC (Gl_Motion_Event), NULL);
  gtk_signal_connect(GTK_OBJECT(gl.glarea), "configure_event",
	GTK_SIGNAL_FUNC(ResizeGL), NULL);
  gtk_signal_connect(GTK_OBJECT(gl.glarea), "expose_event",
	GTK_SIGNAL_FUNC(RedrawGL), NULL);
  gtk_signal_connect (GTK_OBJECT (glwin), "delete_event",
		      GTK_SIGNAL_FUNC (Quit), NULL);
  gtk_signal_connect (GTK_OBJECT (glwin), "destroy",
		      GTK_SIGNAL_FUNC (Quit), NULL);
}

/* EOF */
