/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2003/12/02 12:43:23 $ $Revision: 1.6 $ */
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <stdio.h>
#include "global.h"
#include "light.h"

/* Prototypes */
void FrameOff(GtkWidget *,gpointer);
void Replot_Objects(int);

void Redraw_Light(void)
{
  int a,b;

  glLoadIdentity();
  for (a=0;a<gl.lightmodel.model;a++) {
    glLightModelfv(gl.lightmodel.modelname[a],
		gl.lightmodel.modelparam[a]);
  }
  for (a=0;a<MaxLight;a++) {
    if (gl.light[a].on) {
      for (b=0;b<MaxLightParam;b++) {
	glLightfv(gl.light[a].light, gl.light[a].type[b],
	    gl.light[a].param[b]);
      }
      glEnable(gl.light[a].light);
    } else {
      glDisable(gl.light[a].light);
    }
  }
}

void SetLightParamType(GtkWidget *widget,gpointer data)
{
  int a;
  char s[8];

  a=(int)data;
  if (a>=0) ll.param=(int)data;
  sprintf(s,"%.2lf",gl.light[ll.light].param[ll.param][0]);
  gtk_entry_set_text(GTK_ENTRY(ll.r), s);
  sprintf(s,"%.2lf",gl.light[ll.light].param[ll.param][1]);
  gtk_entry_set_text(GTK_ENTRY(ll.g), s);
  sprintf(s,"%.2lf",gl.light[ll.light].param[ll.param][2]);
  gtk_entry_set_text(GTK_ENTRY(ll.b), s);
  sprintf(s,"%.2lf",gl.light[ll.light].param[ll.param][3]);
  gtk_entry_set_text(GTK_ENTRY(ll.a), s);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ll.active),
      gl.light[ll.light].on);
}

void Plot_Light_Num(GtkWidget *widget,GtkWidget *data)
{
  ll.light=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(data))-1;
  SetLightParamType(NULL,(gpointer)-1);
}

void Change_Value(GtkWidget *widget,gpointer data)
{
  int a;

  a=(int)data;
  gl.light[ll.light].param[ll.param][a]=
    			atof(gtk_entry_get_text(GTK_ENTRY(widget)));
  Replot_Objects(0);
}

void Toggle_Activate(GtkToggleButton *button)
{
  gl.light[ll.light].on=button->active;
  Replot_Objects(0);
}

void Toggle_Move_Light(GtkToggleButton *button)
{
  if (gl.light[ll.light].on) {
    ll.moveit=button->active;
  } else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
  }
}

void CreateLightFrame(GtkWidget *frame)
{
  GtkWidget *vbox;		/* vbox */
  GtkWidget *hbox1;		/* hbox for light controls */
  GtkWidget *hbox2;		/* hbox for light values */
  GtkWidget *label;		/* widget for labels */
  GtkWidget *lightnum;		/* widget for light number */
  GtkWidget *active;		/* if light is activ */
  GtkWidget *r,*g,*b,*a;	/* widgets for parameters */
  GtkWidget *off;		/* switch frame off button */
  GtkWidget *move;		/* move light instead of object */
  GtkWidget *type;		/* parameter to be displayed */
  GtkWidget *menu;		/* menu widget of latter */
  GtkWidget *item;		/* item for the latter */
  GtkAdjustment *adj;		/* adjustment for spin widget */
  char s[8];
  int i;
  int light,param;
  static char *LIGTHPARAMNAME[] = { "Position", "Ambient",
    				    "Diffuse", "Specular" };

  light=0;param=0;
  gl.light[0].on=1;		/* light0 is always on */
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(frame),vbox);
  hbox1 = gtk_hbox_new(FALSE, 0);
  hbox2 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox),hbox1,TRUE,TRUE,3);
  gtk_box_pack_start(GTK_BOX(vbox),hbox2,TRUE,TRUE,3);
  label=gtk_label_new("Light:");
  gtk_box_pack_start(GTK_BOX(hbox1),label,FALSE,FALSE,5);
  gtk_widget_show(label);
  adj = (GtkAdjustment *) gtk_adjustment_new (1.0, 1.0, (double) MaxLight,
      						1.0, 1.0, 0.0);
  lightnum=gtk_spin_button_new(adj,0,0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (lightnum), TRUE);
  gtk_box_pack_start(GTK_BOX(hbox1),lightnum,FALSE,FALSE,5);
  gtk_widget_show(lightnum);
  ll.active=active=gtk_check_button_new_with_label("Active");
  gtk_box_pack_start(GTK_BOX(hbox1),active,FALSE,FALSE,5);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (active), TRUE);
  gtk_widget_show(active);
  move=gtk_check_button_new_with_label("Connect mouse");
  gtk_box_pack_start(GTK_BOX(hbox1),move,FALSE,FALSE,5);
  gtk_widget_show(move);

  off=gtk_check_button_new_with_label("Off");
  gtk_box_pack_end(GTK_BOX(hbox1),off,FALSE,FALSE,0);
  gtk_widget_show(off);

  type=gtk_option_menu_new();
  menu=gtk_menu_new();
  for (i=0;i<MaxLightParam;i++) {
    item=gtk_menu_item_new_with_label(LIGTHPARAMNAME[i]);
    gtk_signal_connect(GTK_OBJECT(item),"activate",
	GTK_SIGNAL_FUNC(SetLightParamType),(gpointer) i);
    gtk_widget_show(item);
    gtk_menu_append (GTK_MENU (menu), item);
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (type), menu);
  gtk_widget_show(menu);
  gtk_widget_show(type);
  gtk_box_pack_start(GTK_BOX(hbox2),type,FALSE,FALSE,5);
  ll.r=r=gtk_entry_new_with_max_length(8);
  gtk_widget_set_usize(r,45,0);
  sprintf(s,"%.2lf",gl.light[light].param[param][0]);
  gtk_entry_set_text(GTK_ENTRY(r), s);
  gtk_box_pack_start(GTK_BOX(hbox2),r,FALSE,FALSE,5);
  gtk_widget_show(r);
  ll.g=g=gtk_entry_new_with_max_length(8);
  gtk_widget_set_usize(g,45,0);
  sprintf(s,"%.2lf",gl.light[light].param[param][1]);
  gtk_entry_set_text(GTK_ENTRY(g), s);
  gtk_box_pack_start(GTK_BOX(hbox2),g,FALSE,FALSE,5);
  gtk_widget_show(g);
  ll.b=b=gtk_entry_new_with_max_length(8);
  gtk_widget_set_usize(b,45,0);
  sprintf(s,"%.2lf",gl.light[light].param[param][2]);
  gtk_entry_set_text(GTK_ENTRY(b), s);
  gtk_box_pack_start(GTK_BOX(hbox2),b,FALSE,FALSE,5);
  gtk_widget_show(b);
  ll.a=a=gtk_entry_new_with_max_length(8);
  gtk_widget_set_usize(a,45,0);
  sprintf(s,"%.2lf",gl.light[light].param[param][3]);
  gtk_entry_set_text(GTK_ENTRY(a), s);
  gtk_box_pack_start(GTK_BOX(hbox2),a,FALSE,FALSE,5);
  gtk_widget_show(a);

  gtk_widget_show(hbox2);
  gtk_widget_show(hbox1);

  gtk_signal_connect (GTK_OBJECT (r), "activate",
     		GTK_SIGNAL_FUNC(Change_Value),(gpointer)0);
  gtk_signal_connect (GTK_OBJECT (g), "activate",
     		GTK_SIGNAL_FUNC(Change_Value),(gpointer)1);
  gtk_signal_connect (GTK_OBJECT (b), "activate",
     		GTK_SIGNAL_FUNC(Change_Value),(gpointer)2);
  gtk_signal_connect (GTK_OBJECT (a), "activate",
     		GTK_SIGNAL_FUNC(Change_Value),(gpointer)3);
  gtk_signal_connect (GTK_OBJECT (active), "toggled",
     		GTK_SIGNAL_FUNC(Toggle_Activate),NULL);
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
     		GTK_SIGNAL_FUNC(Plot_Light_Num),lightnum);
  gtk_signal_connect (GTK_OBJECT (move), "toggled",
	    GTK_SIGNAL_FUNC (Toggle_Move_Light), NULL);
  gtk_signal_connect (GTK_OBJECT (off), "toggled",
	    GTK_SIGNAL_FUNC (FrameOff), (gpointer)frame);
  ll.light=0;
  ll.param=0;
  ll.moveit=0;
  ll.redraw=0;
  gtk_widget_show(vbox);
}
