/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2004/11/24 11:17:03 $ $Revision: 1.22 $ */
/* main */
#include "global.h"
#include <gtk/gtk.h>
#include <gtk/gtkenums.h>
#include <gtk/gtkgl.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "menu.h"
#include "io.h"
#include "about.h"
#include "XPM/icon.xpm"
#ifndef _WIN32
#include <unistd.h>
#endif
#include <getopt.h>

/* Prototypes */
gint AnimateBody(gpointer);
gint Quit(GtkWidget *);
void get_main_menu( GtkWidget  *,GtkWidget **,GtkAccelGroup *);
void reset_gl(GtkWidget *);
void reset_objects(void);
void reset_anim(void);
void reset_io(void);
void reset_movie(void);
void reset_font(void);
void reset_og(void);
void reset_BodyDisplaySettings(void);
void Create_Animation_Buttons(GtkWidget *);
void Create_AnimFrame_Frame(GtkWidget *);
void perr(char *, ...);
void Gl_Window_Events(GtkWidget *);
void Create_Movie_Frame(GtkWidget *);
void CreateEyeFrameItems(GtkWidget *);
void CreateOrientFrameItems(GtkWidget *);
void CreateBGFrameItems(GtkWidget *);
void CreateLightFrame(GtkWidget *);
void CreateObjectFrame(GtkWidget *);
void Reshape_Anim(void);
void io_setfilename(char *);
void io_getfilename(char **);
int Read_rc(char *);
void font_init(const char *, unsigned int, int);
char *my_basename(char *);

void perr(char *format, ...)
{
  va_list arglist;

  va_start(arglist,format);
  vfprintf(stderr,format,arglist);
  va_end(arglist);
  exit(0);
}

int ilog(double f)
{
  int a;
  a=(int)(log10(fabs(f)+(f==0.0)));
  if (a<0) a=2-a;
  else a++;
  if (f<0.0) a++;
  return(a);
}

void Usage(void)
{
  fprintf(stderr,"usage: mozi [-h] [-d] [-f ttf font]");
  fprintf(stderr,"[-b max bodies] [-r] [-w] [-t] ");
  fprintf(stderr,"[moviefile]\n");
  exit(0);
}

int main(int argc, char **argv) 
{  
  GdkGLConfig *glconfig;	/* configuration for GL */
  GtkWidget *glarea;		/* gl container for plotting */
  GtkStyle  *style;		/* style for pixmap background */
  GdkPixmap *pixmap;		/* pixmap for pixmap buttons */
  GdkBitmap *mask;		/* pixmap mask */
  GtkWidget *main_vbox;		/* vbox for menu and everything else */
  GtkWidget *menubar;		/* menubar */
  GtkWidget *animationframe;	/* position frame */
  GtkWidget *moviframe;		/* movie frame */
  GtkWidget *animframeff;	/* animframe frequency frame */
  GtkWidget *eyepositionframe;	/* eye position frame */
  GtkWidget *bgframe;		/* background color frame */
  GtkWidget *orientframe;	/* object orientation frame */
  GtkAccelGroup *accel_group;
  char *fontname;		/* ttf font name */
  char *filename;		/* input filename */
  char *basename;		/* basename of input filename */
  char *header;			/* header text */
  char ch;			/* char for getopt */
  int readrc;			/* read .rc file */

  anim.rcfile=strdup(".mozirc");
  argv0=argv[0];
  gl.outfile=NULL;
  body_alloc=20000;
  readrc=1;
  anim.writerc=0;
  anim.init_show=1;
  anim.groupbytex=0;
  anim.donotdoublebody=0;
  fontname=NULL;
  while ((ch=getopt(argc,argv,"b:df:h-:rstw")) != EOF) {
    switch (ch) {
      case 'b'  : /* body alloc */
		  body_alloc=atoi(optarg);
		  if (body_alloc<1) Usage();
		  break;
      case '-'  : /* it is a long argument of gtk  we skip it */
		  if (strcmp(optarg,"help")==0) Usage();
		  break;
      case 'd' : /* do not remember body save memory */
		  anim.donotdoublebody=1;
		  break;
      case 'f' : /* ttf font file with full path */
		  fontname=strdup(optarg);
		  break;
      case 'r' : /* do not read rc file */
		  readrc=0;
		  break;
      case 's' : /* initially do not show objects */
		  anim.init_show=0;
		  break;
      case 'w' : /* write rc file at the end */
		  anim.writerc=1;
		  break;
      case 't' : /* write rc file at the end */
		  anim.groupbytex=1;
		  break;
      case '?'  :
      case 'h'  : /* help */
		  Usage();
      		  break;
      default   : ;
    }
  }
  reset_io();		/* reset input/output parameters */
  if (gl.outfile==NULL) gl.outfile=strdup("moziparap.prm");
  reset_anim();		/* reset some parameters of animation */
  reset_movie();	/* reset some movie parameter */
  reset_font();		/* reset some font parameter */
  reset_og();		/* reset some constraint parameter */
  if (readrc) readrc=Read_rc(argv[0]);
  if (optind<argc) {	/* filename */
    io_setfilename(argv[optind]);
  } else if (io.filename==NULL) {
    io_setfilename("~/Work/Mozi/Data/demo.dat.gz");
  }
  io_getfilename(&filename);
  basename=my_basename(filename);
  header=(char *)calloc(strlen(basename)+ strlen("GL Display")+20,
						      sizeof(char));
  reset_objects();	/* reset objects parameter */
  gtk_init(&argc, &argv);  
  if (!gdk_gl_query_extension()) {
    perr("GLX extension is not supported by this display.\n");
  }
  glconfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
                                        GDK_GL_MODE_DEPTH |
                                        GDK_GL_MODE_DOUBLE);
  if (glconfig==NULL) {
    perr("No apropriate OpenGL capable visual found\n");
  }
  help.aboutisopen=0;
  help.helpisopen=0;
  /* create two windows: gl and control */
  glwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  sprintf(header,"GL Display (%s)",basename);
  gtk_window_set_title(GTK_WINDOW(glwin), header);
  gtk_window_set_default_size(GTK_WINDOW(glwin), gl.width, gl.height);
  glarea = gtk_drawing_area_new();
  gtk_widget_set_size_request (glarea, 100, 100);
  gtk_widget_set_usize(glarea, 100, 100);
  gtk_widget_set_gl_capability (glarea,glconfig,NULL,TRUE,GDK_GL_RGBA_TYPE);
  gtk_container_add(GTK_CONTAINER(glwin), glarea);


  gl.glarea=glarea;
  gtk_widget_show(glarea);
  gtk_widget_show(glwin);
  {
    GdkGLContext *glcontext = gtk_widget_get_gl_context (glarea);
    GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (glarea);
    if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext)) {
      fprintf(stderr,"mozi: Cannot initialize GL screen\n");
      exit(-1);
    }
    reset_gl(glarea);
    gdk_gl_drawable_gl_end (gldrawable);
  }
  if (fontname==NULL) {
    font_init("/usr/share/fonts/bitstream-vera/VeraBd.ttf",16,0);
  } else {
    font_init(fontname,16,1);
    free(fontname);
  }
  reset_BodyDisplaySettings();

  /* control window */
  ctrlwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  sprintf(header,"Control (%s)",basename);
  gtk_window_set_title(GTK_WINDOW(ctrlwin), header);
  main_vbox = gtk_vbox_new(FALSE, 1);	/* create main vbox */
  gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
  gtk_container_add (GTK_CONTAINER (ctrlwin), main_vbox);
  free(filename);
  free(header);

  /* set icon */
  gtk_widget_realize(ctrlwin);
  gtk_widget_realize(glwin);
  style= gtk_widget_get_style(ctrlwin);
  pixmap = gdk_pixmap_create_from_xpm_d (ctrlwin->window, &mask,
		     &style->bg[GTK_STATE_NORMAL],XPMicon);
  gdk_window_set_icon(ctrlwin->window,NULL,pixmap,mask);
  style= gtk_widget_get_style(glwin);
  pixmap = gdk_pixmap_create_from_xpm_d (glwin->window, &mask,
		     &style->bg[GTK_STATE_NORMAL],XPMicon);
  gdk_window_set_icon(glwin->window,NULL,pixmap,mask);

  /* Menu */
  accel_group = gtk_accel_group_new ();
  get_main_menu(ctrlwin, &menubar,accel_group);
  gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);

  /* Animframe frame */
  menuvars.ffframe=animframeff=gtk_frame_new("Frame frequency");
  gtk_box_pack_start(GTK_BOX(main_vbox), animframeff, FALSE, FALSE, 0);
  gtk_widget_realize(animframeff);
  Create_AnimFrame_Frame(animframeff);

  /* Animation frame */
  animationframe=gtk_frame_new("Animation");
  gtk_box_pack_start(GTK_BOX(main_vbox), animationframe, FALSE, TRUE, 0);
  gtk_widget_realize(animationframe);
  Create_Animation_Buttons(animationframe);

  /* Create movie frame */
  menuvars.movieframe=moviframe=gtk_frame_new("Create movie");
  gtk_box_pack_start(GTK_BOX(main_vbox), moviframe, FALSE, TRUE, 0);
  gtk_widget_realize(moviframe);
  Create_Movie_Frame(moviframe);

  /* Eye position frame */
  menuvars.eyeframe=eyepositionframe=gtk_frame_new("Object position");
  gtk_box_pack_start(GTK_BOX(main_vbox), eyepositionframe, FALSE, TRUE, 0);
  gtk_widget_realize(eyepositionframe);
  CreateEyeFrameItems(eyepositionframe);

  /* Orientation frame */
  menuvars.orientframe=orientframe=gtk_frame_new("Object orientation");
  gtk_box_pack_start(GTK_BOX(main_vbox), orientframe, FALSE, TRUE, 0);
  gtk_widget_realize(orientframe);
  CreateOrientFrameItems(orientframe);

  /* Background color */
  menuvars.bgframe=bgframe=gtk_frame_new("Background color");
  gtk_box_pack_start(GTK_BOX(main_vbox), bgframe, FALSE, TRUE, 0);
  gtk_widget_realize(bgframe);
  CreateBGFrameItems(bgframe);

  /* Light frame */
  menuvars.lightframe=gtk_frame_new("Light");
  gtk_box_pack_start(GTK_BOX(main_vbox),menuvars.lightframe,FALSE,TRUE,0);
  gtk_widget_realize(menuvars.lightframe);
  CreateLightFrame(menuvars.lightframe);
  /* This is hidden at the beginning */

  /* Constraints frame */
  menuvars.objectframe=gtk_frame_new("Constraints");
  gtk_box_pack_start(GTK_BOX(main_vbox),menuvars.objectframe,TRUE,TRUE,0);
  gtk_widget_realize(menuvars.objectframe);
  CreateObjectFrame(menuvars.objectframe);
  menuvars.fontframe=gtk_frame_new("Text");
  gtk_box_pack_start(GTK_BOX(main_vbox),menuvars.fontframe,TRUE,TRUE,0);
  gtk_widget_realize(menuvars.fontframe);
  /* This is shown at the beginning */

  gtk_widget_show(menuvars.objectframe);
  gtk_widget_show(orientframe);
  gtk_widget_show(eyepositionframe);
  gtk_widget_show(animframeff);
  gtk_widget_show(animationframe);
  gtk_widget_show(menubar);
  gtk_widget_show(main_vbox);
  gtk_widget_show(ctrlwin);

  /* if delete event is pressed on any window, we quit */
  gtk_signal_connect (GTK_OBJECT (ctrlwin), "delete_event",
		      GTK_SIGNAL_FUNC (Quit), NULL);
  gtk_signal_connect (GTK_OBJECT (ctrlwin), "destroy",
		      GTK_SIGNAL_FUNC (Quit), NULL);
  Gl_Window_Events(glwin);
  
  /* Start Event Processing Engine */  
  gtk_main ();

  return 0;
}
/* EOF */
